package iterators;

import iterables.MyTableOfThingsRandomIteration;
import java.util.Iterator;


/**
 * 
 * @author J Paul Gibson
 * @version 1
 * Iterates over the String elements in random order<br>
 * 
 *
 */
public class MyTableOfThingsRandomIterator <T> implements Iterator<T> {
	
private int index;
private MyTableOfThingsRandomIteration<T> table;

/**
 * Used to shuffle the indexes into the data array in order to allow sequential random access
 */
private int [] randomindexes;

/**
 * initialise index array so that position i has value i
 * @param length is the size of the indexes array
 */
private void initialise_randomindexes(int length){
	
	randomindexes = new int [length];
	for (int i =0; i<length;i++) randomindexes[i]=i;
}

/**
 * Swap  two index values chosen randomly
 */
private void randomindexswap(){
	
	int maxindex= randomindexes.length;
	
	int randomindex1 = (int) (Math.random()*maxindex);
	int randomindex2 = (int) (Math.random()*maxindex);
	int temp = randomindexes[randomindex1];
	randomindexes[randomindex1] = randomindexes[randomindex2];
	randomindexes[randomindex2] = temp;
}



/**
 * shuffle the index array by calling a sequence of random swaps
 * @param numberOfSwaps is the number of swaps used in the shuffle
 */
private void shuffle_randomindexes(int numberOfSwaps){
	
	if (numberOfSwaps <=0) return;
	else{
		randomindexswap();
		shuffle_randomindexes(numberOfSwaps-1);
	}
}

/**
 * We initialise all private variables, and shuffle the randomindex array
 * @param tab is the table of Strings which we are to iterate over randomly
 */
public MyTableOfThingsRandomIterator(MyTableOfThingsRandomIteration <T> tab) {
index = 0;
table = tab;

initialise_randomindexes(table.length());
shuffle_randomindexes(table.length());

}

/**
 * We initialise all private variables, and shuffle the randomindex array
 * @param tab is the array of Things which we are to iterate over randomly
 */
public MyTableOfThingsRandomIterator( T [] tab) {
index = 0;
table = new MyTableOfThingsRandomIteration<T>(tab);
initialise_randomindexes(table.length());
shuffle_randomindexes(table.length());
}

/**
 * Use the randomindexes array to randomly access an element that has not yet been read
 */
public T next() {
index++;
return (T) table.data[randomindexes[index -1]];
}

public boolean hasNext() {
return index < table.length();
}

/**
 * @throws UnsupportedOperationException as we cannot remove elements while iterating over the collection
 */
public void remove() {
throw new UnsupportedOperationException();
}

}
